@@ -1,15 +1,18 @@
 # frozen_string_literal: true
 
+require_relative "parser/lex_result"
+require_relative "parser/token"
+
 module RBS
   class Parser
-    def self.parse_type(source, range: 0..., variables: [])
+    def self.parse_type(source, range: 0..., variables: [], require_eof: false)
       buf = buffer(source)
-      _parse_type(buf, range.begin || 0, range.end || buf.last_position, variables)
+      _parse_type(buf, range.begin || 0, range.end || buf.last_position, variables, require_eof)
     end
 
-    def self.parse_method_type(source, range: 0..., variables: [])
+    def self.parse_method_type(source, range: 0..., variables: [], require_eof: false)
       buf = buffer(source)
-      _parse_method_type(buf, range.begin || 0, range.end || buf.last_position, variables)
+      _parse_method_type(buf, range.begin || 0, range.end || buf.last_position, variables, require_eof)
     end
 
     def self.parse_signature(source)
@@ -19,6 +22,15 @@ module RBS
       [buf, dirs, decls]
     end
 
+    def self.lex(source)
+      buf = buffer(source)
+      list = _lex(buf, buf.last_position)
+      value = list.map do |type, location|
+        Token.new(type: type, location: location)
+      end
+      LexResult.new(buffer: buf, value: value)
+    end
+
     def self.buffer(source)
       case source
       when String
@@ -28,11 +40,6 @@ module RBS
       end
     end
 
-    autoload :SyntaxError, "rbs/parser_compat/syntax_error"
-    autoload :SemanticsError, "rbs/parser_compat/semantics_error"
-    autoload :LexerError, "rbs/parser_compat/lexer_error"
-    autoload :LocatedValue, "rbs/parser_compat/located_value"
-
     KEYWORDS = %w(
       bool
       bot
