@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module RBS
   module Types
     module NoFreeVariables
@@ -71,7 +73,7 @@ module RBS
           when Types::Bases::Void
             'void'
           when Types::Bases::Any
-            'untyped'
+            raise
           when Types::Bases::Nil
             'nil'
           when Types::Bases::Top
@@ -88,11 +90,32 @@ module RBS
             raise "Unexpected base type: #{inspect}"
           end
         end
+
+        def has_self_type?
+          self.is_a?(Types::Bases::Self)
+        end
+
+        def has_classish_type?
+          self.is_a?(Bases::Instance) || self.is_a?(Bases::Class)
+        end
+
+        def with_nonreturn_void?
+          self.is_a?(Bases::Void)
+        end
       end
 
       class Bool < Base; end
       class Void < Base; end
-      class Any < Base; end
+      class Any < Base
+        def to_s(level=0)
+          @string || "untyped"
+        end
+
+        def todo!
+          @string = '__todo__'
+          self
+        end
+      end
       class Nil < Base; end
       class Top < Base; end
       class Bottom < Base; end
@@ -160,6 +183,18 @@ module RBS
       end
 
       include EmptyEachType
+
+      def has_self_type?
+        false
+      end
+
+      def has_classish_type?
+        false
+      end
+
+      def with_nonreturn_void?
+        false
+      end
     end
 
     class ClassSingleton
@@ -200,6 +235,18 @@ module RBS
           location: location
         )
       end
+
+      def has_self_type?
+        false
+      end
+
+      def has_classish_type?
+        false
+      end
+
+      def with_nonreturn_void?
+        false
+      end
     end
 
     module Application
@@ -239,6 +286,25 @@ module RBS
           enum_for :each_type
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? do |type|
+          if type.is_a?(Bases::Void)
+            # `void` in immediate generics parameter is allowed
+            false
+          else
+            type.with_nonreturn_void?
+          end
+        end
+      end
     end
 
     class Interface
@@ -434,25 +500,49 @@ module RBS
           enum_for :map_type
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Record
-      attr_reader :fields
+      attr_reader :all_fields, :fields, :optional_fields
       attr_reader :location
 
-      def initialize(fields:, location:)
-        @fields = fields
+      def initialize(all_fields: nil, fields: nil, location:)
+        case
+        when fields && all_fields.nil?
+          @all_fields = fields.map { |k, v| [k, [v, true]] }.to_h
+          @fields = fields
+          @optional_fields = {}
+        when all_fields && fields.nil?
+          @all_fields = all_fields
+          @fields = all_fields.filter_map { |k, (v, required)| [k, v] if required }.to_h
+          @optional_fields = all_fields.filter_map { |k, (v, required)| [k, v] unless required }.to_h
+        else
+          raise ArgumentError, "only one of `:fields` or `:all_fields` is requireds"
+        end
+
         @location = location
       end
 
       def ==(other)
-        other.is_a?(Record) && other.fields == fields
+        other.is_a?(Record) && other.fields == fields && other.optional_fields == optional_fields
       end
 
       alias eql? ==
 
       def hash
-        self.class.hash ^ fields.hash
+        self.class.hash ^ all_fields.hash
       end
 
       def free_variables(set = Set.new)
@@ -460,27 +550,35 @@ module RBS
           fields.each_value do |type|
             type.free_variables set
           end
+          optional_fields.each_value do |type|
+            type.free_variables set
+          end
         end
       end
 
       def to_json(state = _ = nil)
-        { class: :record, fields: fields, location: location }.to_json(state)
+        { class: :record, fields: fields, optional_fields: optional_fields, location: location }.to_json(state)
       end
 
       def sub(s)
-        self.class.new(fields: fields.transform_values {|ty| ty.sub(s) },
-                       location: location)
+        self.class.new(
+          all_fields: all_fields.transform_values {|ty, required| [ty.sub(s), required] },
+          location: location
+        )
       end
 
       def to_s(level = 0)
-        return "{ }" if self.fields.empty?
+        return "{ }" if all_fields.empty?
 
-        fields = self.fields.map do |key, type|
-          if key.is_a?(Symbol) && key.match?(/\A[A-Za-z_][A-Za-z_]*\z/)
+        fields = all_fields.map do |key, (type, required)|
+          field = if key.is_a?(Symbol) && key.match?(/\A[A-Za-z_][A-Za-z_0-9]*\z/)
             "#{key}: #{type}"
           else
             "#{key.inspect} => #{type}"
           end
+
+          field = "?#{field}" unless required
+          field
         end
         "{ #{fields.join(", ")} }"
       end
@@ -488,6 +586,7 @@ module RBS
       def each_type(&block)
         if block
           fields.each_value(&block)
+          optional_fields.each_value(&block)
         else
           enum_for :each_type
         end
@@ -495,7 +594,7 @@ module RBS
 
       def map_type_name(&block)
         Record.new(
-          fields: fields.transform_values {|ty| ty.map_type_name(&block) },
+          all_fields: all_fields.transform_values {|ty, required| [ty.map_type_name(&block), required] },
           location: location
         )
       end
@@ -503,13 +602,25 @@ module RBS
       def map_type(&block)
         if block
           Record.new(
-            fields: fields.transform_values {|type| yield type },
+            all_fields: all_fields.transform_values {|type, required| [yield(type), required] },
             location: location
           )
         else
           enum_for :map_type
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Optional
@@ -550,6 +661,8 @@ module RBS
           when Symbol
             return "#{type.to_s(1)} ?"
           end
+        when RBS::Types::Proc
+          return "(#{type.to_s(1)})?"
         end
 
         "#{type.to_s(1)}?"
@@ -580,6 +693,18 @@ module RBS
           enum_for :map_type
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Union
@@ -648,6 +773,18 @@ module RBS
           location: location
         )
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Intersection
@@ -717,6 +854,18 @@ module RBS
           location: location
         )
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Function
@@ -755,10 +904,10 @@ module RBS
 
         def to_s
           if name
-            if Parser::KEYWORDS.include?(name.to_s)
-              "#{type} `#{name}`"
-            else
+            if name.match?(/\A[a-zA-Z0-9_]+\z/)
               "#{type} #{name}"
+            else
+              "#{type} `#{name}`"
             end
           else
             "#{type}"
@@ -843,12 +992,12 @@ module RBS
       def map_type(&block)
         if block
           Function.new(
-            required_positionals: required_positionals.map {|param| param.map_type(&block) },
-            optional_positionals: optional_positionals.map {|param| param.map_type(&block) },
+            required_positionals: amap(required_positionals) {|param| param.map_type(&block) },
+            optional_positionals: amap(optional_positionals) {|param| param.map_type(&block) },
             rest_positionals: rest_positionals&.yield_self {|param| param.map_type(&block) },
-            trailing_positionals: trailing_positionals.map {|param| param.map_type(&block) },
-            required_keywords: required_keywords.transform_values {|param| param.map_type(&block) },
-            optional_keywords: optional_keywords.transform_values {|param| param.map_type(&block) },
+            trailing_positionals: amap(trailing_positionals) {|param| param.map_type(&block) },
+            required_keywords: hmapv(required_keywords) {|param| param.map_type(&block) },
+            optional_keywords: hmapv(optional_keywords) {|param| param.map_type(&block) },
             rest_keywords: rest_keywords&.yield_self {|param| param.map_type(&block) },
             return_type: yield(return_type)
           )
@@ -857,6 +1006,22 @@ module RBS
         end
       end
 
+      def amap(array, &block)
+        if array.empty?
+          _ = array
+        else
+          array.map(&block)
+        end
+      end
+
+      def hmapv(hash, &block)
+        if hash.empty?
+          _ = hash
+        else
+          hash.transform_values(&block)
+        end
+      end
+
       def map_type_name(&block)
         map_type do |type|
           type.map_type_name(&block)
@@ -1015,69 +1180,199 @@ module RBS
           false
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        if each_param.any? {|param| param.type.with_nonreturn_void? }
+          true
+        else
+          if return_type.is_a?(Bases::Void)
+            false
+          else
+            return_type.with_nonreturn_void?
+          end
+        end
+      end
+    end
+
+    class UntypedFunction
+      attr_reader :return_type
+
+      def initialize(return_type:)
+        @return_type = return_type
+      end
+
+      def free_variables(acc = Set.new)
+        return_type.free_variables(acc)
+      end
+
+      def map_type(&block)
+        if block
+          update(return_type: yield(return_type))
+        else
+          enum_for :map_type
+        end
+      end
+
+      def each_type(&block)
+        if block
+          yield return_type
+        else
+          enum_for :each_type
+        end
+      end
+
+      def each_param(&block)
+        if block
+          # noop
+        else
+          enum_for :each_param
+        end
+      end
+
+      def to_json(state = _ = nil)
+        {
+          return_type: return_type
+        }.to_json(state)
+      end
+
+      def sub(subst)
+        map_type { _1.sub(subst) }
+      end
+
+      def with_return_type(ty)
+        update(return_type: ty)
+      end
+
+      def update(return_type: self.return_type)
+        UntypedFunction.new(return_type: return_type)
+      end
+
+      def empty?
+        true
+      end
+
+      def has_self_type?
+        return_type.has_self_type?
+      end
+
+      def has_classish_type?
+        return_type.has_classish_type?
+      end
+
+      def with_nonreturn_void?
+        false
+      end
+
+      def param_to_s
+        "?"
+      end
+
+      def return_to_s
+        return_type.to_s(1)
+      end
+
+      def ==(other)
+        other.is_a?(UntypedFunction) && other.return_type == return_type
+      end
+
+      alias eql? ==
+
+      def hash
+        self.class.hash ^ return_type.hash
+      end
+
     end
 
     class Block
       attr_reader :type
       attr_reader :required
+      attr_reader :self_type
 
-      def initialize(type:, required:)
+      def initialize(type:, required:, self_type: nil)
         @type = type
         @required = required ? true : false
+        @self_type = self_type
       end
 
       def ==(other)
         other.is_a?(Block) &&
           other.type == type &&
-          other.required == required
+          other.required == required &&
+          other.self_type == self_type
       end
 
       def to_json(state = _ = nil)
         {
           type: type,
-          required: required
+          required: required,
+          self_type: self_type
         }.to_json(state)
       end
 
       def sub(s)
         self.class.new(
           type: type.sub(s),
-          required: required
+          required: required,
+          self_type: self_type&.sub(s)
         )
       end
 
       def map_type(&block)
         Block.new(
           required: required,
-          type: type.map_type(&block)
+          type: type.map_type(&block),
+          self_type: self_type ? yield(self_type) : nil
         )
       end
     end
 
+    module SelfTypeBindingHelper
+      module_function
+
+      def self_type_binding_to_s(t)
+        if t
+          "[self: #{t}] "
+        else
+          ""
+        end
+      end
+    end
+
     class Proc
       attr_reader :type
       attr_reader :block
+      attr_reader :self_type
       attr_reader :location
 
-      def initialize(location:, type:, block:)
+      def initialize(location:, type:, block:, self_type: nil)
         @type = type
         @block = block
         @location = location
+        @self_type = self_type
       end
 
       def ==(other)
-        other.is_a?(Proc) && other.type == type && other.block == block
+        other.is_a?(Proc) && other.type == type && other.block == block && other.self_type == self_type
       end
 
       alias eql? ==
 
       def hash
-        self.class.hash ^ type.hash ^ block.hash
+        self.class.hash ^ type.hash ^ block.hash ^ self_type.hash
       end
 
       def free_variables(set = Set[])
         type.free_variables(set)
         block&.type&.free_variables(set)
+        self_type&.free_variables(set)
         set
       end
 
@@ -1086,24 +1381,33 @@ module RBS
           class: :proc,
           type: type,
           block: block,
-          location: location
+          location: location,
+          self_type: self_type
         }.to_json(state)
       end
 
       def sub(s)
-        self.class.new(type: type.sub(s), block: block&.sub(s), location: location)
+        self.class.new(
+          type: type.sub(s),
+          block: block&.sub(s),
+          self_type: self_type&.sub(s),
+          location: location
+        )
       end
 
       def to_s(level = 0)
+        self_binding = SelfTypeBindingHelper.self_type_binding_to_s(self_type)
+        block_self_binding = SelfTypeBindingHelper.self_type_binding_to_s(block&.self_type)
+
         case
         when b = block
           if b.required
-            "^(#{type.param_to_s}) { (#{b.type.param_to_s}) -> #{b.type.return_to_s} } -> #{type.return_to_s}"
+            "^(#{type.param_to_s}) #{self_binding}{ (#{b.type.param_to_s}) #{block_self_binding}-> #{b.type.return_to_s} } -> #{type.return_to_s}"
           else
-            "^(#{type.param_to_s}) ?{ (#{b.type.param_to_s}) -> #{b.type.return_to_s} } -> #{type.return_to_s}"
+            "^(#{type.param_to_s}) #{self_binding}?{ (#{b.type.param_to_s}) #{block_self_binding}-> #{b.type.return_to_s} } -> #{type.return_to_s}"
           end
         else
-          "^(#{type.param_to_s}) -> #{type.return_to_s}"
+          "^(#{type.param_to_s}) #{self_binding}-> #{type.return_to_s}"
         end
       end
 
@@ -1111,6 +1415,9 @@ module RBS
         if block
           type.each_type(&block)
           self.block&.type&.each_type(&block)
+          if self_type = self.block&.self_type
+            yield self_type
+          end
         else
           enum_for :each_type
         end
@@ -1120,6 +1427,7 @@ module RBS
         Proc.new(
           type: type.map_type_name(&block),
           block: self.block&.map_type {|type| type.map_type_name(&block) },
+          self_type: self_type&.map_type_name(&block),
           location: location
         )
       end
@@ -1129,12 +1437,33 @@ module RBS
           Proc.new(
             type: type.map_type(&block),
             block: self.block&.map_type(&block),
+            self_type: self_type ? yield(self_type) : nil,
             location: location
           )
         else
           enum_for :map_type
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        if type.with_nonreturn_void?
+          true
+        else
+          if block = block()
+            block.type.with_nonreturn_void? || block.self_type&.with_nonreturn_void? || false
+          else
+            false
+          end
+        end
+      end
     end
 
     class Literal
@@ -1168,6 +1497,47 @@ module RBS
       def to_s(level = 0)
         literal.inspect
       end
+
+      def has_self_type?
+        false
+      end
+
+      def has_classish_type?
+        false
+      end
+
+      def with_nonreturn_void?
+        false
+      end
+
+      TABLE = {
+        "\\a" => "\a",
+        "\\b" => "\b",
+        "\\e" => "\033",
+        "\\f" => "\f",
+        "\\n" => "\n",
+        "\\r" => "\r",
+        "\\s" => " ",
+        "\\t" => "\t",
+        "\\v" => "\v",
+        "\\\"" => "\"",
+        "\\\'" => "'",
+        "\\\\" => "\\",
+        "\\" => ""
+      }
+
+      def self.unescape_string(string, is_double_quote)
+        if is_double_quote
+          string.gsub!(/\\([0-9]{1,3})/) { ($1 || "").to_i(8).chr }
+          string.gsub!(/\\x([0-9a-f]{1,2})/) { ($1 || "").to_i(16).chr }
+          string.gsub!(/\\u([0-9a-fA-F]{4})/) { ($1 || "").to_i(16).chr(Encoding::UTF_8) }
+          string.gsub!(/\\[abefnrstv"'\\]?/, TABLE)
+          string
+        else
+          string.gsub!(/\\['\\]/, TABLE)
+          string
+        end
+      end
     end
   end
 end
