@@ -238,6 +238,13 @@ EOF
 
         assert_raises(RBS::RecursiveAliasDefinitionError) do
           builder.build_interface(type_name("::_I3"))
+        end.tap do |error|
+          assert_equal <<~DETAILED_MESSAGE, error.detailed_message if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::RecursiveAliasDefinitionError)
+
+                alias a b
+                ^^^^^^^^^
+          DETAILED_MESSAGE
         end
 
         assert_raises(RBS::DuplicatedMethodDefinitionError) do
@@ -572,16 +579,34 @@ end
         assert_raises(InvalidVarianceAnnotationError) { builder.build_instance(type_name("::Test1")) }.tap do |error|
           assert_equal :X, error.param.name
           assert_equal "C[X]", error.location.source
+          assert_equal <<~DETAILED_MESSAGE, error.detailed_message if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::InvalidVarianceAnnotationError)
+
+              class Test1[in X] < C[X]
+                                  ^^^^
+          DETAILED_MESSAGE
         end
 
         assert_raises(InvalidVarianceAnnotationError) { builder.build_instance(type_name("::Test2")) }.tap do |error|
           assert_equal :X, error.param.name
           assert_equal "include M[X]", error.location.source
+          assert_equal <<~DETAILED_MESSAGE, error.detailed_message if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::InvalidVarianceAnnotationError)
+
+                include M[X]
+                ^^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
 
         assert_raises(InvalidVarianceAnnotationError) { builder.build_instance(type_name("::Test3")) }.tap do |error|
           assert_equal :X, error.param.name
           assert_equal "include _I[X]", error.location.source
+          assert_equal <<~DETAILED_MESSAGE, error.detailed_message if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::InvalidVarianceAnnotationError)
+
+                include _I[X]
+                ^^^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
       end
     end
@@ -614,11 +639,23 @@ end
         assert_raises(InvalidVarianceAnnotationError) { builder.build_instance(type_name("::Test1")) }.tap do |error|
           assert_equal :X, error.param.name
           assert_equal "(X) -> void", error.location.source
+          assert_equal <<~DETAILED_MESSAGE, error.detailed_message if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::InvalidVarianceAnnotationError)
+
+                def foo: (X) -> void
+                         ^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
 
         assert_raises(InvalidVarianceAnnotationError) { builder.build_instance(type_name("::Test2")) }.tap do |error|
           assert_equal :X, error.param.name
           assert_equal "attr_reader x: X", error.location.source
+          assert_equal <<~DETAILED_MESSAGE, error.detailed_message if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::InvalidVarianceAnnotationError)
+
+                attr_reader x: X
+                ^^^^^^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
       end
     end
@@ -971,6 +1008,13 @@ EOF
 
         assert_raises UnknownMethodAliasError do
           builder.build_singleton(type_name("::Error"))
+        end.tap do |error|
+          assert_equal error.detailed_message, <<~DETAILED_MESSAGE if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::UnknownMethodAliasError)
+
+                alias self.xxx self.yyy
+                ^^^^^^^^^^^^^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
       end
     end
@@ -1044,13 +1088,17 @@ EOF
           assert_instance_of Definition, definition
 
           assert_method_definition definition.methods[:instance_reader], ["() -> ::String"]
+          assert_equal definition.methods[:instance_reader].method_types.first.location.source, "attr_reader instance_reader: String"
           assert_ivar_definition definition.instance_variables[:@instance_reader], "::String"
 
           assert_method_definition definition.methods[:instance_writer=], ["(::Integer instance_writer) -> ::Integer"]
+          assert_equal definition.methods[:instance_writer=].method_types.first.location.source, "attr_writer instance_writer(@writer): Integer"
           assert_ivar_definition definition.instance_variables[:@writer], "::Integer"
 
           assert_method_definition definition.methods[:instance_accessor], ["() -> ::Symbol"]
+          assert_equal definition.methods[:instance_accessor].method_types.first.location.source, "attr_accessor instance_accessor(): Symbol"
           assert_method_definition definition.methods[:instance_accessor=], ["(::Symbol instance_accessor) -> ::Symbol"]
+          assert_equal definition.methods[:instance_accessor=].method_types.first.location.source, "attr_accessor instance_accessor(): Symbol"
           assert_nil definition.instance_variables[:@instance_accessor]
         end
       end
@@ -1073,13 +1121,17 @@ EOF
           assert_instance_of Definition, definition
 
           assert_method_definition definition.methods[:reader], ["() -> ::String"]
+          assert_equal definition.methods[:reader].method_types.first.location.source, "attr_reader self.reader: String"
           assert_ivar_definition definition.instance_variables[:@reader], "::String"
 
           assert_method_definition definition.methods[:writer=], ["(::Integer writer) -> ::Integer"]
+          assert_equal definition.methods[:writer=].method_types.first.location.source, "attr_writer self.writer(@writer): Integer"
           assert_ivar_definition definition.instance_variables[:@writer], "::Integer"
 
           assert_method_definition definition.methods[:accessor], ["() -> ::Symbol"]
+          assert_equal definition.methods[:accessor].method_types.first.location.source, "attr_accessor self.accessor(): Symbol"
           assert_method_definition definition.methods[:accessor=], ["(::Symbol accessor) -> ::Symbol"]
+          assert_equal definition.methods[:accessor=].method_types.first.location.source, "attr_accessor self.accessor(): Symbol"
           assert_nil definition.instance_variables[:@accessor]
         end
       end
@@ -1090,7 +1142,19 @@ EOF
     SignatureManager.new do |manager|
       manager.files[Pathname("foo.rbs")] = <<EOF
 class Hello
+  public
+
   def initialize: (String) -> void
+  def initialize_copy: (self) -> self
+  def initialize_clone: (self) -> self
+  def initialize_dup: (self) -> self
+  def respond_to_missing?: () -> bool
+
+  def self.initialize: (String) -> void
+  def self.initialize_copy: (self) -> self
+  def self.initialize_clone: (self) -> self
+  def self.initialize_dup: (self) -> self
+  def self.respond_to_missing?: () -> bool
 end
 EOF
 
@@ -1100,11 +1164,20 @@ EOF
         builder.build_instance(type_name("::Hello")).tap do |definition|
           assert_instance_of Definition, definition
           assert_method_definition definition.methods[:initialize], ["(::String) -> void"], accessibility: :private
+          assert_method_definition definition.methods[:initialize_copy], ["(self) -> self"], accessibility: :private
+          assert_method_definition definition.methods[:initialize_clone], ["(self) -> self"], accessibility: :private
+          assert_method_definition definition.methods[:initialize_dup], ["(self) -> self"], accessibility: :private
+          assert_method_definition definition.methods[:respond_to_missing?], ["() -> bool"], accessibility: :private
         end
 
         builder.build_singleton(type_name("::Hello")).yield_self do |definition|
           assert_instance_of Definition, definition
           assert_method_definition definition.methods[:new], ["(::String) -> ::Hello"], accessibility: :public
+          assert_method_definition definition.methods[:initialize], ["(::String) -> void"], accessibility: :public
+          assert_method_definition definition.methods[:initialize_copy], ["(self) -> self"], accessibility: :public
+          assert_method_definition definition.methods[:initialize_clone], ["(self) -> self"], accessibility: :public
+          assert_method_definition definition.methods[:initialize_dup], ["(self) -> self"], accessibility: :public
+          assert_method_definition definition.methods[:respond_to_missing?], ["() -> bool"], accessibility: :public
         end
       end
     end
@@ -1535,6 +1608,13 @@ end
 
         assert_raises RBS::InvalidOverloadMethodError do
           builder.build_instance(type_name("::Hello"))
+        end.tap do |error|
+          assert_equal error.detailed_message, <<~DETAILED_MESSAGE if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::InvalidOverloadMethodError)
+
+                def foo: (Integer) -> String | ...
+                ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
       end
     end
@@ -1666,6 +1746,13 @@ end
 
         assert_raises RBS::DuplicatedInterfaceMethodDefinitionError do
           builder.build_instance(type_name("::Hello"))
+        end.tap do |error|
+          assert_equal error.detailed_message, <<~DETAILED_MESSAGE if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::DuplicatedInterfaceMethodDefinitionError)
+
+                include _I2
+                ^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
       end
     end
@@ -1689,6 +1776,13 @@ end
 
         assert_raises RBS::DuplicatedMethodDefinitionError do
           builder.build_instance(type_name("::Hello"))
+        end.tap do |error|
+          assert_equal error.detailed_message, <<~DETAILED_MESSAGE if Exception.method_defined?(:detailed_message)
+            #{error.message} (RBS::DuplicatedMethodDefinitionError)
+
+                def foo: () -> String
+                ^^^^^^^^^^^^^^^^^^^^^
+          DETAILED_MESSAGE
         end
       end
     end
@@ -2234,12 +2328,12 @@ end
       definition_builder = RBS::DefinitionBuilder.new(env: env.resolve_type_names)
       definition_builder.build_instance(TypeName("::Foo")).tap do |defn|
         defn.methods[:request].tap do |m|
-          assert_equal ["(::Object::name name, *untyped args) ?{ (*untyped) -> untyped } -> untyped"], m.method_types.map(&:to_s)
+          assert_equal ["(::interned name, *untyped, **untyped) ?{ (?) -> untyped } -> untyped"], m.method_types.map(&:to_s)
         end
       end
       definition_builder.build_instance(TypeName("::Mod")).tap do |defn|
         defn.methods[:request].tap do |m|
-          assert_equal ["(::Object::name name, *untyped args) ?{ (*untyped) -> untyped } -> untyped"], m.method_types.map(&:to_s)
+          assert_equal ["(::interned name, *untyped, **untyped) ?{ (?) -> untyped } -> untyped"], m.method_types.map(&:to_s)
         end
       end
   end
@@ -2501,4 +2595,114 @@ end
       end
     end
   end
+
+  def test_extend_overload
+    SignatureManager.new do |manager|
+      manager.files[Pathname("foo.rbs")] = <<~EOF
+        module M
+          def f: () -> Integer
+        end
+        class A
+          extend M
+          def self.f: () -> String | ...
+        end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_singleton(type_name("::A")).tap do |definition|
+          assert_instance_of Definition, definition
+
+          definition.methods[:f].tap do |method|
+            assert_equal [TypeName("::A"), TypeName("::M")], method.defs.map(&:defined_in)
+            assert_equal [TypeName("::A"), TypeName("::A")], method.defs.map(&:implemented_in)
+          end
+        end
+      end
+    end
+  end
+
+  def test_module_alias__superclass
+    SignatureManager.new do |manager|
+      manager.add_file("foo.rbs", <<-EOF)
+module Foo
+  class Bar
+  end
+end
+
+module Baz = Foo
+
+class Hoge < Baz::Bar
+end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::Hoge"))
+        builder.build_singleton(type_name("::Hoge"))
+      end
+    end
+  end
+
+  def test_module_alias__mixin
+    SignatureManager.new do |manager|
+      manager.add_file("foo.rbs", <<-EOF)
+module Foo
+end
+
+module Bar = Foo
+
+class Baz
+  include Bar
+  include Bar
+end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::Baz"))
+        builder.build_singleton(type_name("::Baz"))
+      end
+    end
+  end
+
+  def test_module_alias__module_self
+    SignatureManager.new do |manager|
+      manager.add_file("foo.rbs", <<-EOF)
+class Foo = Integer
+
+module Bar : Foo
+end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::Bar"))
+        builder.build_singleton(type_name("::Bar"))
+      end
+    end
+  end
+
+  def test_alias__to_module_self_indierect_method
+    SignatureManager.new(system_builtin: false) do |manager|
+      manager.add_file("foo.rbs", <<-EOF)
+module Kernel
+  alias foo __id__
+end
+
+module Foo
+end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::Foo"))
+      end
+    end
+  end
 end
