@@ -626,6 +626,14 @@ RBS
       assert_equal "?foo, ?foo: String", method_type.type.param_to_s
     end
 
+    RBS::Parser.parse_method_type(buffer("(`foo`: String) -> void")).tap do |method_type|
+      assert_equal "`foo`: String", method_type.type.param_to_s
+    end
+
+    RBS::Parser.parse_method_type(buffer("(?`foo`: String) -> void")).tap do |method_type|
+      assert_equal "?`foo`: String", method_type.type.param_to_s
+    end
+
     RBS::Parser.parse_method_type(buffer("(1) -> void")).tap do |method_type|
       assert_equal "1", method_type.type.param_to_s
     end
@@ -633,7 +641,7 @@ RBS
     assert_raises RBS::ParsingError do
       RBS::Parser.parse_method_type(buffer("(foo + 1) -> void"))
     end.tap do |exn|
-      assert_equal "test.rbs:1:7...1:8: Syntax error: unexpected token for method type parameters, token=`1` (tINTEGER)", exn.message
+      assert_equal "test.rbs:1:5...1:6: Syntax error: unexpected token for function parameter name, token=`+` (tOPERATOR)", exn.message
     end
 
     assert_raises RBS::ParsingError do
@@ -642,6 +650,18 @@ RBS
       assert_equal "test.rbs:1:15...1:18: Syntax error: required keyword argument type is expected, token=`Bar` (tUIDENT)", exn.message
     end
 
+    assert_raises RBS::ParsingError do
+      RBS::Parser.parse_method_type(buffer("(foo`: untyped) -> void"))
+    end.tap do |exn|
+      assert_equal "test.rbs:1:4...1:5: Syntax error: unexpected token for function parameter name, token=``` (tOPERATOR)", exn.message
+    end
+
+    assert_raises RBS::ParsingError do
+      RBS::Parser.parse_method_type(buffer("(?foo\": untyped) -> void"))
+    end.tap do |exn|
+      assert_equal "test.rbs:1:5...1:6: Syntax error: unexpected token for function parameter name, token=`\"` (ErrorToken)", exn.message
+    end
+
     assert_raises RBS::ParsingError do
       RBS::Parser.parse_method_type(buffer("(**untyped, ?Bar) -> void"))
     end.tap do |exn|
@@ -649,6 +669,44 @@ RBS
     end
   end
 
+  def test_duplicate_keyword
+    RBS::Parser.parse_method_type(buffer("(top foo, foo: top) -> void")).tap do |method_type|
+      assert_equal "top foo, foo: top", method_type.type.param_to_s
+    end
+
+    RBS::Parser.parse_method_type(buffer("(?foo, foo: top) -> void")).tap do |method_type|
+      assert_equal "?foo, foo: top", method_type.type.param_to_s
+    end
+
+    RBS::Parser.parse_method_type(buffer("(foo: top, **top foo) -> void")).tap do |method_type|
+      assert_equal "foo: top, **top foo", method_type.type.param_to_s
+    end
+
+    assert_raises RBS::ParsingError do
+      RBS::Parser.parse_method_type(buffer("(foo: top, foo: top) -> void"))
+    end.tap do |exn|
+      assert_equal "test.rbs:1:11...1:14: Syntax error: duplicated keyword argument, token=`foo` (tLIDENT)", exn.message
+    end
+
+    assert_raises RBS::ParsingError do
+      RBS::Parser.parse_method_type(buffer("(foo: top, ?foo: top) -> void"))
+    end.tap do |exn|
+      assert_equal "test.rbs:1:12...1:15: Syntax error: duplicated keyword argument, token=`foo` (tLIDENT)", exn.message
+    end
+
+    assert_raises RBS::ParsingError do
+      RBS::Parser.parse_method_type(buffer("(?foo: top, foo: top) -> void"))
+    end.tap do |exn|
+      assert_equal "test.rbs:1:12...1:15: Syntax error: duplicated keyword argument, token=`foo` (tLIDENT)", exn.message
+    end
+
+    assert_raises RBS::ParsingError do
+      RBS::Parser.parse_method_type(buffer("(?foo: top, ?foo: top) -> void"))
+    end.tap do |exn|
+      assert_equal "test.rbs:1:13...1:16: Syntax error: duplicated keyword argument, token=`foo` (tLIDENT)", exn.message
+    end
+  end
+
   def test_parse_method_type2
     RBS::Parser.parse_method_type(buffer("(foo?: String, bar!: Integer) -> void")).tap do |method_type|
       assert_equal "foo?: String, bar!: Integer", method_type.type.param_to_s
@@ -697,6 +755,12 @@ RBS
     end
   end
 
+  def test_negative_range
+    assert_raises ArgumentError do
+      RBS::Parser.parse_type("a", range: -2...-1)
+    end
+  end
+
   def test_parse_eof_nil
     code = buffer("type1   ")
 
@@ -713,4 +777,63 @@ RBS
       assert_nil type
     end
   end
+
+  def test_parse_require_eof
+    RBS::Parser.parse_type("String", range: 0..., require_eof: false)
+    assert_raises(RBS::ParsingError) do
+      RBS::Parser.parse_type("String void", range: 0..., require_eof: true)
+    end
+
+    RBS::Parser.parse_method_type("() -> void () -> void", range: 0..., require_eof: false)
+    assert_raises(RBS::ParsingError) do
+      RBS::Parser.parse_method_type("() -> void () -> void", range: 0..., require_eof: true)
+    end
+  end
+
+  def test_proc__untyped_function
+    RBS::Parser.parse_type("^(?) -> Integer").tap do |type|
+      assert_instance_of RBS::Types::UntypedFunction, type.type
+    end
+
+    RBS::Parser.parse_type("^() { (?) -> String } -> Integer").tap do |type|
+      assert_instance_of RBS::Types::UntypedFunction, type.block.type
+    end
+  end
+
+  def test_proc__untyped_function_parse_error
+    assert_raises(RBS::ParsingError) do
+      RBS::Parser.parse_type("^(?) { (?) -> void } -> Integer")
+    end
+  end
+
+  def test__lex
+    content = <<~RBS
+      # LineComment
+      class Foo[T < Integer] < Bar # Comment
+      end
+    RBS
+    tokens = RBS::Parser._lex(buffer(content), content.length)
+    assert_equal [:tLINECOMMENT, '# LineComment', 0...13], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, "\n", 13...14], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:kCLASS, 'class', 14...19], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, " ", 19...20], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tUIDENT, 'Foo', 20...23], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:pLBRACKET, '[', 23...24], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tUIDENT, 'T', 24...25], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, " ", 25...26], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:pLT, '<', 26...27], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, " ", 27...28], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tUIDENT, 'Integer', 28...35], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:pRBRACKET, ']', 35...36], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, " ", 36...37], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:pLT, '<', 37...38], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, " ", 38...39], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tUIDENT, 'Bar', 39...42], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, " ", 42...43], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tCOMMENT, '# Comment', 43...52], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, "\n", 52...53], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:kEND, 'end', 53...56], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:tTRIVIA, "\n", 56...57], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+    assert_equal [:pEOF, '', 57...57], tokens.shift.then { |t| [t[0], t[1].source, t[1].range] }
+  end
 end
