@@ -1157,7 +1157,7 @@ EOF
   def test_parsing_quoted_method_name
     Parser.parse_signature(<<-EOF).yield_self do |foo_decl,|
 module Kernel
-  def `: (Symbol | String name) -> String?
+  def `: (interned name) -> String?
 
   # Returns a `Binding` object, describing the variable and method bindings
   # at the point of call. This object can be used when calling `eval` to
@@ -2011,7 +2011,7 @@ end
 
   def test_generics_bound
     Parser.parse_signature(<<-EOF).tap do |_, _, decls|
-class Foo[X < _Each[Y], Y]
+class Foo[X < _Each[Y]?, Y]
   def foo: [X < Array[Y]] (X) -> X
 end
     EOF
@@ -2023,14 +2023,15 @@ end
           assert_equal :X, param.name
           assert_equal :invariant, param.variance
           refute_predicate param, :unchecked?
-          assert_equal parse_type("_Each[Y]", variables: [:Y]), param.upper_bound
+          assert_nil param.upper_bound
+          assert_equal parse_type("_Each[Y]?", variables: [:Y]), param.upper_bound_type
         end
 
         decl.type_params[1].tap do |param|
           assert_equal :Y, param.name
           assert_equal :invariant, param.variance
           refute_predicate param, :unchecked?
-          assert_nil param.upper_bound
+          assert_nil param.upper_bound_type
         end
 
         decl.members[0].tap do |member|
@@ -2039,18 +2040,41 @@ end
             assert_equal :invariant, param.variance
             refute_predicate param, :unchecked?
             assert_equal parse_type("Array[Y]", variables: [:Y]), param.upper_bound
+            assert_equal parse_type("Array[Y]", variables: [:Y]), param.upper_bound_type
           end
         end
       end
     end
   end
 
-  def test_generics_bound_error
-    assert_raises RBS::ParsingError do
-      Parser.parse_signature(<<-EOF)
-        class Foo[X < string]
+  def test_generics_default
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
+class Foo[X < _Each[String]? = Array[String]]
+end
+    EOF
+      decls[0].tap do |decl|
+        assert_instance_of Declarations::Class, decl
+
+        assert_equal 1, decl.type_params.size
+        decl.type_params[0].tap do |param|
+          assert_equal :X, param.name
+          assert_equal :invariant, param.variance
+          refute_predicate param, :unchecked?
+          assert_nil param.upper_bound
+          assert_equal 29...44, param.location[:default].range
+          assert_equal parse_type("_Each[String]?"), param.upper_bound_type
+          assert_equal parse_type("Array[String]"), param.default_type
         end
-            EOF
+      end
+    end
+
+    assert_raises do
+      Parser.parse_signature(<<~EOF)
+        class Foo[X = untyped, Y]
+        end
+      EOF
+    end.tap do |error|
+      assert_match(/required type parameter is not allowed after optional type parameter/, error.message)
     end
   end
 
@@ -2106,17 +2130,33 @@ end
 
       dirs[0].tap do |use|
         assert_equal 1, use.clauses.size
-        assert_equal TypeName("RBS::Namespace"), use.clauses[0].type_name
-        assert_equal :NS, use.clauses[0].new_name
+
+        use.clauses[0].tap do |clause|
+          assert_equal TypeName("RBS::Namespace"), clause.type_name
+          assert_equal :NS, clause.new_name
+          assert_equal "RBS::Namespace as NS", clause.location.source
+          assert_equal "RBS::Namespace", clause.location[:type_name].source
+          assert_equal "as", clause.location[:keyword].source
+          assert_equal "NS", clause.location[:new_name].source
+        end
       end
 
       dirs[1].tap do |use|
         assert_equal 2, use.clauses.size
 
-        assert_equal TypeName("RBS::TypeName"), use.clauses[0].type_name
-        assert_nil use.clauses[0].new_name
+        use.clauses[0].tap do |clause|
+          assert_equal TypeName("RBS::TypeName"), clause.type_name
+          assert_nil clause.new_name
+          assert_equal "RBS::TypeName", clause.location[:type_name].source
+          assert_nil clause.location[:keyword]
+          assert_nil clause.location[:new_name]
+        end
 
-        assert_equal Namespace("RBS::AST::Declarations::"), use.clauses[1].namespace
+        use.clauses[1].tap do |clause|
+          assert_equal Namespace("RBS::AST::Declarations::"), clause.namespace
+          assert_equal "RBS::AST::Declarations::", clause.location[:namespace].source
+          assert_equal "*", clause.location[:star].source
+        end
       end
     end
   end
