@@ -1,7 +1,7 @@
 require_relative "test_helper"
 
 class ModuleSingletonTest < Test::Unit::TestCase
-  include TypeAssertions
+  include TestHelper
 
   testing "singleton(::Module)"
 
@@ -19,7 +19,7 @@ class ModuleSingletonTest < Test::Unit::TestCase
 end
 
 class ModuleInstanceTest < Test::Unit::TestCase
-  include TypeAssertions
+  include TestHelper
 
   testing "::Module"
 
@@ -27,6 +27,20 @@ class ModuleInstanceTest < Test::Unit::TestCase
     BAR = 1
   end
 
+  def test_include
+    assert_send_type "(Module) -> Module",
+                     Module.new, :include, Module.new
+    assert_send_type "(Module, Module) -> Module",
+                     Module.new, :include, Module.new, Module.new
+  end
+
+  def test_prepend
+    assert_send_type "(Module) -> Module",
+                     Module.new, :prepend, Module.new
+    assert_send_type "(Module, Module) -> Module",
+                     Module.new, :prepend, Module.new, Module.new
+  end
+
   def test_refine
     assert_send_type "(Module) { () -> void } -> Refinement",
                      Foo, :refine, String do nil end
@@ -74,6 +88,36 @@ class ModuleInstanceTest < Test::Unit::TestCase
                      Foo, :module_eval do nil end
   end
 
+  def test_module_function
+    mod = Module.new do
+      def foo; end
+
+      def bar; end
+    end
+
+    assert_send_type(
+      "() -> nil",
+      mod, :module_function
+    )
+    assert_send_type(
+      "(Symbol) -> Symbol",
+      mod, :module_function, :foo
+    )
+    assert_send_type(
+      "(String) -> String",
+      mod, :module_function, "foo"
+    )
+    assert_send_type(
+      "(ToStr) -> ToStr",
+      mod, :module_function, ToStr.new("foo")
+    )
+
+    assert_send_type(
+      "(Symbol, String) -> Array[Symbol | String]",
+      mod, :module_function, :foo, "bar"
+    )
+  end
+
   def test_class_eval
     assert_send_type "(String) -> nil",
                      Foo, :class_eval, 'nil'
@@ -86,6 +130,11 @@ class ModuleInstanceTest < Test::Unit::TestCase
                      Foo, :class_eval do nil end
   end
 
+  def test_class_exec
+    assert_send_type '(*String) { (*String) [self: Foo] -> Integer } -> Integer',
+                      Foo, :class_exec, '1', '2' do |*x| x.join.to_i end
+  end
+
   def test_alias_method
     mod = Module.new do
       def foo
@@ -124,9 +173,81 @@ class ModuleInstanceTest < Test::Unit::TestCase
     )
 
     assert_send_type(
-      "(Symbol, String) -> Array[Symbol | String]",
+      "(Symbol, String) -> Array[interned]",
       mod, :private, :foo, "bar"
     )
+
+    assert_send_type(
+      "(Array[Symbol | String]) -> Array[Symbol | String]",
+      mod, :private, [:foo, "bar"]
+    )
+  end
+
+  def test_private_class_method
+    mod = Module.new do
+      def self.foo; end
+      def self.bar; end
+    end
+
+    assert_send_type(
+      "(Symbol) -> Module",
+      mod, :private_class_method, :foo
+    )
+
+    assert_send_type(
+      "(Symbol, Symbol) -> Module",
+      mod, :private_class_method, :foo, :bar
+    )
+
+    assert_send_type(
+      "(String) -> Module",
+      mod, :private_class_method, "foo"
+    )
+
+    assert_send_type(
+      "(String, String) -> Module",
+      mod, :private_class_method, "foo", "bar"
+    )
+
+    assert_send_type(
+      "(Array[Symbol | String]) -> Module",
+      mod, :private_class_method, [:foo, "bar"]
+    )
+  end
+
+  def test_protected
+    mod = Module.new do
+      def foo; end
+
+      def bar; end
+    end
+
+    assert_send_type(
+      "() -> nil",
+      mod, :protected
+    )
+    assert_send_type(
+      "(Symbol) -> Symbol",
+      mod, :protected, :foo
+    )
+    assert_send_type(
+      "(String) -> String",
+      mod, :protected, "foo"
+    )
+    assert_send_type(
+      "(ToStr) -> ToStr",
+      mod, :protected, ToStr.new("foo")
+    )
+
+    assert_send_type(
+      "(Symbol, String) -> Array[Symbol | String]",
+      mod, :protected, :foo, "bar"
+    )
+
+    assert_send_type(
+      "(Array[Symbol | String]) -> Array[Symbol | String]",
+      mod, :protected, [:foo, "bar"]
+    )
   end
 
   def test_public
@@ -154,16 +275,53 @@ class ModuleInstanceTest < Test::Unit::TestCase
     )
 
     assert_send_type(
-      "(Symbol, String) -> Array[Symbol | String]",
+      "(Symbol, String) -> Array[interned]",
       mod, :public, :foo, "bar"
     )
+
+    assert_send_type(
+      "(Array[Symbol | String]) -> Array[Symbol | String]",
+      mod, :public, [:foo, "bar"]
+    )
+  end
+
+  def test_public_class_method
+    mod = Module.new do
+      def self.foo; end
+      def self.bar; end
+    end
+
+    assert_send_type(
+      "(Symbol) -> Module",
+      mod, :public_class_method, :foo
+    )
+
+    assert_send_type(
+      "(Symbol, Symbol) -> Module",
+      mod, :public_class_method, :foo, :bar
+    )
+
+    assert_send_type(
+      "(String) -> Module",
+      mod, :public_class_method, "foo"
+    )
+
+    assert_send_type(
+      "(String, String) -> Module",
+      mod, :public_class_method, "foo", "bar"
+    )
+
+    assert_send_type(
+      "(Array[Symbol | String]) -> Module",
+      mod, :public_class_method, [:foo, "bar"]
+    )
   end
 
   def test_attr
     if RUBY_VERSION >= '3.0'
       mod = Module.new
       assert_send_type(
-        "(*Symbol | String arg0) -> Array[Symbol]",
+        "(*interned arg0) -> Array[Symbol]",
         mod, :attr, :foo
       )
     end
@@ -261,4 +419,20 @@ class ModuleInstanceTest < Test::Unit::TestCase
       )
     end
   end
+
+  def test_set_temporary_name
+    mod = Module.new
+
+    with_string "fake_name" do |name|
+      assert_send_type(
+        "(::string) -> ::Module",
+        mod, :set_temporary_name, name
+      )
+    end
+
+    assert_send_type(
+      "(nil) -> Module",
+      mod, :set_temporary_name, nil
+    )
+  end
 end
