@@ -270,7 +270,7 @@ class TimeTest < StdlibTest
 end
 
 class TimeSingletonTest < Test::Unit::TestCase
-  include TypeAssertions
+  include TestHelper
   testing "singleton(::Time)"
 
   def test_now
@@ -343,8 +343,27 @@ class TimeSingletonTest < Test::Unit::TestCase
   end
 end
 
+class TimeInstanceTest < Test::Unit::TestCase
+  include TestHelper
+  testing "::Time"
+
+  def test_to_a
+    assert_send_type "() -> [ Integer, Integer, Integer, Integer, Integer, Integer, Integer, Integer, bool, String ]",
+                     Time.now, :to_a
+    assert_send_type "() -> [ Integer, Integer, Integer, Integer, Integer, Integer, Integer, Integer, bool, nil ]",
+                     Time.now(in: 0), :to_a
+  end
+
+  def test_zone
+    assert_send_type "() -> String",
+                     Time.now, :zone
+    assert_send_type "() -> nil",
+                     Time.now(in: 0), :zone
+  end
+end
+
 class TimeInDateTest < Test::Unit::TestCase
-  include TypeAssertions
+  include TestHelper
 
   require "date"
 
